package laboratoire4;

public class MiniMax {
    private static final int MAX_DEPTH = 1;

    public static MiniMaxResult miniMax(PusherBoard board) {
        MovingBoard game = new MovingBoard(board.getBoard(), board.getPlayer());
        return miniMax(game, true, 0, Integer.MIN_VALUE, Integer.MAX_VALUE);
    }

    private static MiniMaxResult miniMax(MovingBoard game, boolean max, int depth, int alpha, int beta) {
        depth += 1;

        if (max) {
            return max(game, depth, alpha, beta);
        }

        return min(game, depth, alpha, beta);
    }

    private static MiniMaxResult max(MovingBoard game, int depth, int alpha, int beta) {
        int alphaT = Integer.MIN_VALUE;
        MovingPawn lastPawn = null;
        Pawn.PawnMovement lastMovement = null;

        for (MovingPawn pawn : game.getMaxPawns()) {
            for (Pawn.PawnMovement movement : Pawn.PawnMovement.values()) {
                if (!pawn.isMoveValid(game.getBoard(), movement)) {
                    continue;
                }

                lastPawn = pawn;
                lastMovement = movement;

                game.move(pawn, movement);

                int score = evaluate(pawn, game, depth);
                if (depth < MAX_DEPTH) {
                    score = miniMax(game, false, depth, Math.max(alpha, alphaT), beta).getScore();
                }

                game.revertMove();

                if (score > alphaT) {
                    alphaT = score;
                }

                if (alphaT >= beta) {
                    return new MiniMaxResult(alphaT, pawn.getRow(), pawn.getCol(), movement);
                }
            }
        }

        if (lastPawn == null) {
            return new MiniMaxResult(alphaT, 0, 0, null);
        }
        return new MiniMaxResult(alphaT, lastPawn.getRow(), lastPawn.getCol(), lastMovement);
    }

    private static MiniMaxResult min(MovingBoard game, int depth, int alpha, int beta) {
        int betaT = Integer.MAX_VALUE;
        MovingPawn lastPawn = null;
        Pawn.PawnMovement lastMovement = null;

        for (MovingPawn pawn : game.getMinPawns()) {
            for (Pawn.PawnMovement movement : Pawn.PawnMovement.values()) {
                if (!pawn.isMoveValid(game.getBoard(), movement)) {
                    continue;
                }

                lastPawn = pawn;
                lastMovement = movement;

                game.move(pawn, movement);

                int score = evaluate(pawn, game, depth);
                if (depth < MAX_DEPTH) {
                    score = miniMax(game, true, depth, alpha, Math.min(betaT, beta)).getScore();
                }

                game.revertMove();

                if (score < betaT) {
                    betaT = score;
                }

                if (betaT <= beta) {
                    return new MiniMaxResult(betaT, pawn.getRow(), pawn.getCol(), movement);
                }
            }
        }

        if (lastPawn == null) {
            return new MiniMaxResult(betaT, 0, 0, null);
        }
        return new MiniMaxResult(betaT, lastPawn.getRow(), lastPawn.getCol(), lastMovement);
    }

    private static int evaluate(IPawn pawn, MovingBoard game, int depth) {
        int score = didWin(pawn) + didCapture(pawn, game);

        score += pawn.isPusher() ? 0 : 5;

        int row = pawn.getRow();
        int col = pawn.getCol();
        int nextNextRow = row + pawn.getDirection() * 2;

        // Attire nos pusher vers nos pushed
        if (pawn.isPusher() && nextNextRow >= 0 && nextNextRow <= 7) {
            for (Pawn.PawnMovement move : Pawn.PawnMovement.values()) {
                int nextCol = col + move.getMove();
                if (nextCol < 0 || nextCol > 7) {
                    continue;
                }

                IPawn nearPawn = game.getBoard()[nextNextRow][nextCol];
                if (nearPawn != null && !nearPawn.isPusher() && nearPawn.getPlayer() == pawn.getPlayer()) {
                    score += 5;
                }
            }
        }

        return score * pawn.getDirection();
    }

    private static int didWin(IPawn pawn) {
        if (pawn.getRow() == pawn.getPlayer().getGoal()) {
            return 100;
        }

        return 0;
    }

    private static int didCapture(IPawn pawn, MovingBoard game) {
        IPawn capturedPawn = game.getBoard()[pawn.getRow()][pawn.getCol()];

        if (capturedPawn != null && capturedPawn.getPlayer() != game.getPlayer()) {
            return 500;
        }

        return 0;
    }

    static class MiniMaxResult {
        private final int score;
        private final int row;
        private final int col;
        private final Pawn.PawnMovement movement;

        public MiniMaxResult(int score, int row, int col, Pawn.PawnMovement movement) {
            this.score = score;
            this.row = row;
            this.col = col;
            this.movement = movement;
        }

        public int getScore() {
            return score;
        }

        public int getRow() {
            return row;
        }

        public int getCol() {
            return col;
        }

        public Pawn.PawnMovement getMovement() {
            return movement;
        }

        @Override
        public String toString() {
            return "MiniMaxResult{" +
                    "score=" + score +
                    ", col=" + col +
                    ", row=" + row +
                    ", movement=" + movement +
                    '}';
        }
    }
}
